@@ -26,7 +26,7 @@ where
     state: Arc<State>,
     connection_handler: ConnectionHandler<State>,
     connection_condition: ConnectionCondition<State>,
-    websocket_handler: WebsocketHandler<State>,
+    websocket_handler: Box<dyn WebsocketHandler<State>>,
 }
 
 /// Represents a function able to handle a connection.
@@ -35,7 +35,7 @@ pub type ConnectionHandler<State> = fn(
     TcpStream,
     Arc<Vec<RouteHandler<State>>>,
     Arc<ErrorHandler>,
-    Arc<WebsocketHandler<State>>,
+    Arc<Box<dyn WebsocketHandler<State>>>,
     Arc<State>,
 );
 
@@ -43,10 +43,11 @@ pub type ConnectionHandler<State> = fn(
 pub type ConnectionCondition<State> = fn(&mut TcpStream, Arc<State>) -> bool;
 
 /// Represents a function able to handle a WebSocket handshake and consequent data frames.
-pub type WebsocketHandler<State> = fn(Request, TcpStream, Arc<State>);
+pub trait WebsocketHandler<State>: Fn(Request, TcpStream, Arc<State>) + Send + Sync {}
+impl<T, S> WebsocketHandler<S> for T where T: Fn(Request, TcpStream, Arc<S>) + Send + Sync {}
 
 /// Represents a function able to handle a request.
-/// It is passed a the request as well as the app's state, and must return a response.
+/// It is passed the request as well as the app's state, and must return a response.
 ///
 /// ## Example
 /// The most basic request handler would be as follows:
@@ -58,6 +59,37 @@ pub type WebsocketHandler<State> = fn(Request, TcpStream, Arc<State>);
 pub trait RequestHandler<State>: Fn(Request, Arc<State>) -> Response + Send + Sync {}
 impl<T, S> RequestHandler<S> for T where T: Fn(Request, Arc<S>) -> Response + Send + Sync {}
 
+/// Represents a function able to handle a request.
+/// It is passed only the request, and must return a response.
+/// If you want access to the app's state, consider using the `RequestHandler` trait instead.
+///
+/// ## Example
+/// The most basic stateless request handler would be as follows:
+/// ```
+/// fn handler(request: Request) -> Response {
+///     Response::new(StatusCode::OK, b"Success", &request)
+/// }
+/// ```
+pub trait StatelessRequestHandler<State>: Fn(Request) -> Response + Send + Sync {}
+impl<T, S> StatelessRequestHandler<S> for T where T: Fn(Request) -> Response + Send + Sync {}
+
+/// Represents a function able to handle a request with respect to the route it was called from.
+/// It is passed the request, the app's state, and the route it was called from, and must return a response.
+///
+/// ## Example
+/// The most basic path-aware request handler would be as follows:
+/// ```
+/// fn handler(request: Request, _: Arc<()>, route: &str) -> Response {
+///     Response::new(StatusCode::OK, format!("Success matching route {}", route), &request)
+/// }
+/// ```
+#[rustfmt::skip]
+pub trait PathAwareRequestHandler<State>:
+    Fn(Request, Arc<State>, &str) -> Response + Send + Sync {}
+#[rustfmt::skip]
+impl<T, S> PathAwareRequestHandler<S> for T where
+    T: Fn(Request, Arc<S>, &str) -> Response + Send + Sync {}
+
 /// Represents a function able to handle an error.
 /// The first parameter of type `Option<Request>` will be `Some` if the request could be parsed.
 /// Otherwise, it will be `None` and the status code will be `StatusCode::BadRequest`.
@@ -109,7 +141,7 @@ where
             state: Arc::new(State::default()),
             connection_handler: client_handler,
             connection_condition: |_, _| true,
-            websocket_handler: |_, _, _| (),
+            websocket_handler: Box::new(|_, _, _| ()),
         }
     }
 
@@ -122,7 +154,7 @@ where
             state: Arc::new(state),
             connection_handler: client_handler,
             connection_condition: |_, _| true,
-            websocket_handler: |_, _, _| (),
+            websocket_handler: Box::new(|_, _, _| ()),
         }
     }
 
@@ -188,6 +220,42 @@ where
         self
     }
 
+    /// Adds a route and associated handler to the server.
+    /// Does not pass the state to the handler.
+    /// Routes can include wildcards, for example `/blog/*`.
+    ///
+    /// If you want to access the app's state in the handler, consider using `with_route`.
+    ///
+    /// ## Panics
+    /// This function will panic if the route string cannot be converted to a `Uri` object.
+    pub fn with_stateless_route<T>(mut self, route: &str, handler: T) -> Self
+    where
+        T: StatelessRequestHandler<State> + 'static,
+    {
+        self.routes.push(RouteHandler {
+            route: route.parse().unwrap(),
+            handler: Box::new(move |request, _| handler(request)),
+        });
+        self
+    }
+
+    /// Adds a path-aware route and associated handler to the server.
+    /// Routes can include wildcards, for example `/blog/*`.
+    /// Will also pass the route to the handler at runtime.
+    ///
+    /// ## Panics
+    /// This function will panic if the route string cannot be converted to a `Uri` object.
+    pub fn with_path_aware_route<T>(mut self, route: &'static str, handler: T) -> Self
+    where
+        T: PathAwareRequestHandler<State> + 'static,
+    {
+        self.routes.push(RouteHandler {
+            route: route.parse().unwrap(),
+            handler: Box::new(move |request, state| handler(request, state, route)),
+        });
+        self
+    }
+
     /// Sets the error handler for the server.
     pub fn with_error_handler(mut self, handler: ErrorHandler) -> Self {
         self.error_handler = handler;
@@ -203,8 +271,11 @@ where
 
     /// Sets the websocket handler, a function which processes WebSocket handshakes.
     /// This is passed the stream, state, and the request which triggered its calling.
-    pub fn with_websocket_handler(mut self, handler: WebsocketHandler<State>) -> Self {
-        self.websocket_handler = handler;
+    pub fn with_websocket_handler<T>(mut self, handler: T) -> Self
+    where
+        T: WebsocketHandler<State> + 'static,
+    {
+        self.websocket_handler = Box::new(handler);
         self
     }
 
@@ -217,8 +288,8 @@ where
 
     /// Gets a reference to the app's state.
     /// This should only be used in the main thread, as the state is passed to request handlers otherwise.
-    pub fn get_state(&self) -> &Arc<State> {
-        &self.state
+    pub fn get_state(&self) -> Arc<State> {
+        self.state.clone()
     }
 }
 
@@ -229,7 +300,7 @@ fn client_handler<State>(
     mut stream: TcpStream,
     routes: Arc<Vec<RouteHandler<State>>>,
     error_handler: Arc<ErrorHandler>,
-    websocket_handler: Arc<WebsocketHandler<State>>,
+    websocket_handler: Arc<Box<dyn WebsocketHandler<State>>>,
     state: Arc<State>,
 ) {
     let addr = stream.peer_addr().unwrap();
